package de.oose.trainings.oot.junit.geometrie;

/**
 * <p>berschrift: Geometrie-Beispiel JUnit</p>
 * <p>Beschreibung: Trainingsbeispiel OOT/Junit</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Organisation: oose.de GmbH</p>
 * @author Uwe Vigenschow
 * @version 2.0
 */

public class Dreieck {

  private double kanteA;
  private double kanteB;
  private double kanteC;

  private double delta = 0.001;

  public Dreieck(double kanteA, double kanteB, double kanteC) {
    if (checkDreieck(kanteA, kanteB, kanteC)){
      this.kanteA = kanteA;
      this.kanteB = kanteB;
      this.kanteC = kanteC;
    }
    else{
      this.kanteA = 1.0;
      this.kanteB = 1.0;
      this.kanteC = 1.0;
    }

    invariant();
  }

  public void setKanten(double kanteA, double kanteB, double kanteC) {
    if (checkDreieck(kanteA, kanteB, kanteC)){
      this.kanteA = kanteA;
      this.kanteB = kanteB;
      this.kanteC = kanteC;
    }
    invariant();
  }

  public double getKanteA(){
    return kanteA;
  }
  public double getKanteB(){
    return kanteB;
  }
  public double getKanteC(){
    return kanteC;
  }

  public static boolean checkDreieck(double a, double b, double c){
    // Zwei Seiten muessen zusammen stets laenger sein als die Dritte!
    return ((a + b > c) && (a + c > b) && (b + c > a));
  }

  private void invariant(){
    assert(checkDreieck(kanteA, kanteB, kanteC));
  }

  private boolean gleich(double a, double b){
    if (Math.abs(a-b) < delta){
      return true;
    }
    else {
      return false;
    }
  }

  public double flaeche(){
    // Flaechenberechnung nach Heron
    double s = (kanteA+kanteB+kanteC)/2.0; // halber Umfang

    double f = s*(s-kanteA) * (s-kanteB) * (s-kanteC);
    f = Math.sqrt(f);

    return f;
  }

  public boolean isGleichseitig(){
    if (gleich(kanteA, kanteB) && gleich(kanteB, kanteC)){
      return true;
    }
    else {
      return false;
    }
  }

  public boolean isGleichschenklig(){
    if (isGleichseitig()) return false;
    if (gleich(kanteA, kanteB) ||
        gleich(kanteB, kanteC) ||
        gleich(kanteA, kanteC)){
      return true;
    }
    else {
      return false;
    }
  }

}