package vertrag;

import junit.framework.*;

public class TestVertrag extends TestCase {
  private static final double delta = 0.001;

  private Vertrag vertrag = null;
  private KundenDummy kunde = null;
  private MitarbeiterDummy vermittler = null;
  private ProviRechnerMock proviRechner = null;

  protected void setUp() throws Exception {
    super.setUp();
    kunde = new KundenDummy("Paul Meyer");
    vermittler = new MitarbeiterDummy("Heinz", "Mller");
    proviRechner = new ProviRechnerMock();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
    kunde = null;
    vermittler = null;
    proviRechner = null;
  }

  public void testAngelegtPositiv() {
    vertrag = new Vertrag(1, kunde, vermittler, proviRechner);  // positiver Grenzwert beim Prfen
    assertEquals(1, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag = null;

    vertrag = new Vertrag(0, kunde, vermittler, proviRechner);  // negativer Grenzwert beim Prfen
    assertEquals(0, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag = null;
  }

  public void testAngelegtNegativ() {
    vertrag = new Vertrag(7, kunde, vermittler, proviRechner);  // beliebiger Zwischenwert
    assertEquals(7, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.abrechnen();  //nicht erlaubt
    assertEquals(7, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.aktivieren();  //nicht erlaubt
    assertEquals(7, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.ruhen();  //nicht erlaubt
    assertEquals(7, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.beenden();  //nicht erlaubt
    assertEquals(7, vertrag.getNummer());
    assertEquals("Paul Meyer",vertrag.getKunden());
    assertEquals("Heinz Mller", vertrag.getVermittler());
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag = null;
  }


  public void testPruefenAbgelehntPositiv() {
    vertrag = new Vertrag(1, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertTrue(vertrag.isAktiv());
    assertEquals(0.0, vertrag.getProvision(), delta);
    vertrag = null;

    vertrag = new Vertrag(0, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());
    vertrag = null;
  }

  public void testPruefenNegativ() {
    vertrag = new Vertrag(7, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);

    vertrag.pruefen();  //nicht nochmal erlaubt
    assertEquals(7, vertrag.getNummer());
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.aktivieren();  //nicht erlaubt
    assertEquals(7, vertrag.getNummer());
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag = null;
  }

  public void testAbgelehntNegativ() {
    vertrag = new Vertrag(0, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.abrechnen();  //nicht erlaubt
    assertEquals(0, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.aktivieren();  //nicht erlaubt
    assertEquals(0, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.pruefen();  //nicht erlaubt
    assertEquals(0, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.ruhen();  //nicht erlaubt
    assertEquals(0, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.beenden();  //nicht erlaubt
    assertEquals(0, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGELEHNT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;
  }

  public void testAbrechnenPositiv() {
    vertrag = new Vertrag(1, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    vertrag.abrechnen();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag = null;
  }

  public void testAbrechnenNegativ() {
    vertrag = new Vertrag(2, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    vertrag.abrechnen();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.abrechnen();  //nicht nochmal erlaubt
    assertEquals(2, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.pruefen();  //nicht erlaubt
    assertEquals(2, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag.aktivieren();  //nicht erlaubt
    assertEquals(2, vertrag.getNummer());
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag = null;
  }

  public void testRuhenPositiv() {
    vertrag = new Vertrag(1, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag.ruhen();
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;

    vertrag = new Vertrag(3, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    vertrag.abrechnen();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag.ruhen();
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;
  }

  public void testRuhenNegativ() {
    vertrag = new Vertrag(4, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    vertrag.ruhen();
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.pruefen();  //nicht erlaubt
    assertEquals(4, vertrag.getNummer());
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.abrechnen();  //nicht erlaubt
    assertEquals(4, vertrag.getNummer());
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.ruhen();  //nicht nochmal erlaubt
    assertEquals(4, vertrag.getNummer());
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;
  }

  public void testBeendenPositiv() {
    vertrag = new Vertrag(1, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertTrue(vertrag.isAktiv());
    vertrag.beenden();
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;

    vertrag = new Vertrag(5, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    vertrag.abrechnen();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag.beenden();
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;
  }

  public void testBeendenNegativ() {
    vertrag = new Vertrag(32000, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    vertrag.abrechnen();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag.beenden();
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.beenden();  //nicht nochmal erlaubt
    assertEquals(32000, vertrag.getNummer());
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.pruefen();  //nicht erlaubt
    assertEquals(32000, vertrag.getNummer());
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.abrechnen();  //nicht erlaubt
    assertEquals(32000, vertrag.getNummer());
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.ruhen();  //nicht erlaubt
    assertEquals(32000, vertrag.getNummer());
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag.aktivieren();  //nicht erlaubt
    assertEquals(32000, vertrag.getNummer());
    assertEquals(vertrag.VZ_BEENDET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());

    vertrag = null;
  }

  public void testAktivieren() {
  // es fehlen nur noch die Ablaufketten aus dem Zustand RUHEND wieder zurueck!
    vertrag = new Vertrag(1, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertTrue(vertrag.isAktiv());
    vertrag.ruhen();
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertTrue(!vertrag.isAktiv());
    vertrag.aktivieren();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    assertEquals(0.0, vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag = null;

    vertrag = new Vertrag(3, kunde, vermittler, proviRechner);
    assertEquals(vertrag.VZ_ANGELEGT, vertrag.getZustand());
    vertrag.pruefen();
    assertEquals(vertrag.VZ_GEPRUEFT, vertrag.getZustand());
    vertrag.abrechnen();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());
    vertrag.ruhen();
    assertEquals(vertrag.VZ_RUHEND, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(!vertrag.isAktiv());
    vertrag.aktivieren();
    assertEquals(vertrag.VZ_ABGERECHNET, vertrag.getZustand());
    assertEquals(proviRechner.getLetztenWert(), vertrag.getProvision(), delta);
    assertTrue(vertrag.isAktiv());

    vertrag = null;
  }
}

