package vertrag;

/**
 * <p>berschrift: Vertrag-Beispiel JUnit</p>
 * <p>Beschreibung: Trainingsbeispiel OOT/Junit</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Organisation: oose.de GmbH</p>
 * @author Uwe Vigenschow
 * @version 1.0
 * Einfache Implementierung ohne Java-Spezialitaeten
 */

public class Vertrag {
  public static final int VZ_UNGUELTIG = 0;
  public static final int VZ_ANGELEGT = 1;
  public static final int VZ_GEPRUEFT = 2;
  public static final int VZ_ABGERECHNET = 3;
  public static final int VZ_ABGELEHNT = 4;
  public static final int VZ_RUHEND = 5;
  public static final int VZ_BEENDET = 6;

  // zum Historisieren aktiver Zustnde fr ruhen() und aktivieren()
  private int historyState;

  private interface Zustand {
    abstract public int pruefen();
    abstract public int provisionieren();
    abstract public int ruhen();
    abstract public int aktivieren();
    abstract public int beenden();
    abstract public int getZustand();
  }

  private class Angelegt implements Zustand {
    public int pruefen() {
      if (getNummer() > 0) {
        return VZ_GEPRUEFT;
      } else{
        return VZ_ABGELEHNT;
      }
    }

    public int provisionieren() {
      return VZ_UNGUELTIG;
    }
    public int ruhen() {
      return VZ_UNGUELTIG;
    }
    public int aktivieren() {
      return VZ_UNGUELTIG;
    }
    public int beenden() {
      return VZ_UNGUELTIG;
    }
    public int getZustand() {
      return VZ_ANGELEGT;
    }
  }

  private class Geprueft implements Zustand {
    public int pruefen() {
      return VZ_UNGUELTIG;
    }
    public int provisionieren() {
      return VZ_ABGERECHNET;
    }
    public int ruhen() {
      historyState = getZustand();
      return VZ_RUHEND;
    }
    public int aktivieren() {
      return VZ_UNGUELTIG;
    }
    public int beenden() {
      return VZ_BEENDET;
    }
    public int getZustand() {
      return VZ_GEPRUEFT;
    }
  }

  private class Abgerechnet implements Zustand {
    public int pruefen() {
      return VZ_UNGUELTIG;
    }
    public int provisionieren() {
      return VZ_UNGUELTIG;
    }
    public int ruhen() {
      historyState = getZustand();
      return VZ_RUHEND;
    }
    public int aktivieren() {
      return VZ_UNGUELTIG;
    }
    public int beenden() {
      return VZ_BEENDET;
    }
    public int getZustand() {
      return VZ_ABGERECHNET;
    }
  }

  private class Abgelehnt implements Zustand {
    public int pruefen() {
      return VZ_UNGUELTIG;
    }
    public int provisionieren() {
      return VZ_UNGUELTIG;
    }
    public int ruhen() {
      return VZ_UNGUELTIG;
    }
    public int aktivieren() {
      return VZ_UNGUELTIG;
    }
    public int beenden() {
      return VZ_UNGUELTIG;
    }
    public int getZustand() {
      return VZ_ABGELEHNT;
    }
  }
  private class Ruhend implements Zustand {
    public int pruefen() {
      return VZ_UNGUELTIG;
    }
    public int provisionieren() {
      return VZ_UNGUELTIG;
    }
    public int ruhen() {
      return VZ_UNGUELTIG;
    }
    public int aktivieren() {
      return historyState;   // historisierten Zustand wieder einnehmen
    }
    public int beenden() {
      return VZ_UNGUELTIG;
    }
    public int getZustand() {
      return VZ_RUHEND;
    }
  }
  private class Beendet implements Zustand {
    public int pruefen() {
      return VZ_UNGUELTIG;
    }
    public int provisionieren() {
      return VZ_UNGUELTIG;
    }
    public int ruhen() {
      return VZ_UNGUELTIG;
    }
    public int aktivieren() {
      return VZ_UNGUELTIG;
    }
    public int beenden() {
      return VZ_UNGUELTIG;
    }
    public int getZustand() {
      return VZ_BEENDET;
    }
  }


  private long nummer;
  private Zustand myZustand;

  public Vertrag(long nr) {
    myZustand = new Angelegt();
    nummer = nr;
    setStatus(VZ_ANGELEGT);
    historyState = VZ_UNGUELTIG;
  }

  public long getNummer(){
    return nummer;
  }

  public boolean pruefen() {
    setStatus(myZustand.pruefen());
    if (VZ_GEPRUEFT == myZustand.getZustand()) {
      return true;
    }
    return false;
  }

  public void abrechnen() {
    setStatus(myZustand.provisionieren());
  }

  public void ruhen() {
    setStatus(myZustand.ruhen());
  }

  public void aktivieren() {
    setStatus(myZustand.aktivieren());
  }

  public void beenden() {
    setStatus(myZustand.beenden());
  }

  public int getZustand() {
    if (null != myZustand) {
      return myZustand.getZustand();
    } else {
      return VZ_UNGUELTIG;
    }
  }

  public boolean isAktiv() {
    if ((VZ_ANGELEGT == getZustand()) ||
        (VZ_GEPRUEFT == getZustand()) ||
        (VZ_ABGERECHNET == getZustand())) {
      return true;
    } else {
      return false;
    }
  }

  private void setStatus(int neuerZustand) {
    if (VZ_UNGUELTIG != neuerZustand) {
      if (neuerZustand != myZustand.getZustand()) {  // ein neuer Zustand?
        myZustand = null;           // alten Zustand frei geben
        switch (neuerZustand) {     // Zielzustand anlegen
          case VZ_ANGELEGT: myZustand = new Angelegt();
            break;
          case VZ_GEPRUEFT: myZustand = new Geprueft();
            break;
          case VZ_ABGERECHNET: myZustand = new Abgerechnet();
            break;
          case VZ_ABGELEHNT: myZustand = new Abgelehnt();
            break;
          case VZ_RUHEND: myZustand = new Ruhend();
            break;
          case VZ_BEENDET: myZustand = new Beendet();
            break;
          default: myZustand = null;
            break;
        }
      }
    }
  }

}