package vertrag;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * <p>berschrift: Vertrag-Beispiel JUnit</p>
 * <p>Beschreibung: Trainingsbeispiel OOT/Junit</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Organisation: oose.de GmbH</p>
 * @author Uwe Vigenschow
 * @version 1.1
 * Implementierung mit Java-Sepzialitaeten
 */

public class Vertrag {
  public static final Class VZ_UNGUELTIG = Ungueltig.class;
  public static final Class VZ_ANGELEGT = Angelegt.class;
  public static final Class VZ_GEPRUEFT = Geprueft.class;
  public static final Class VZ_ABGERECHNET = Abgerechnet.class;
  public static final Class VZ_ABGELEHNT = Abgelehnt.class;
  public static final Class VZ_RUHEND = Ruhend.class;
  public static final Class VZ_BEENDET = Beendet.class;

  // zum Historisieren aktiver Zustnde fr ruhen() und aktivieren()
  private Class historyState;

  private interface Zustand {
    Class pruefen();
    Class provisionieren();
    Class ruhen();
    Class aktivieren();
    Class beenden();
    Class getZustand();
  }

  private class Ungueltig implements Zustand {
    public Ungueltig() {
      // damit es luft
    }
    public Class pruefen() {
      return VZ_UNGUELTIG;
    }
    public Class provisionieren() {
      return VZ_UNGUELTIG;
    }
    public Class ruhen() {
      return VZ_UNGUELTIG;
    }
    public Class aktivieren() {
      return VZ_UNGUELTIG;
    }
    public Class beenden() {
      return VZ_UNGUELTIG;
    }
    public Class getZustand() {
      return VZ_UNGUELTIG;
    }
  }

  private class Angelegt implements Zustand {
    public Angelegt() {
      // damit es luft
    }

    public Class pruefen() {
      if (getNummer() > 0) {
        return VZ_GEPRUEFT;
      } else{
        return VZ_ABGELEHNT;
      }
    }

    public Class provisionieren() {
      return VZ_ANGELEGT;
    }
    public Class ruhen() {
      return VZ_ANGELEGT;
    }
    public Class aktivieren() {
      return VZ_ANGELEGT;
    }
    public Class beenden() {
      return VZ_ANGELEGT;
    }
    public Class getZustand() {
      return VZ_ANGELEGT;
    }
  }

  private class Geprueft implements Zustand {
    public Class pruefen() {
      return VZ_GEPRUEFT;
    }
    public Class provisionieren() {
      return VZ_ABGERECHNET;
    }
    public Class ruhen() {
      historyState = getZustand();
      return VZ_RUHEND;
    }
    public Class aktivieren() {
      return VZ_GEPRUEFT;
    }
    public Class beenden() {
      return VZ_BEENDET;
    }
    public Class getZustand() {
      return VZ_GEPRUEFT;
    }
  }

  private class Abgerechnet implements Zustand {
    public Class pruefen() {
      return VZ_ABGERECHNET;
    }
    public Class provisionieren() {
      return VZ_ABGERECHNET;
    }
    public Class ruhen() {
      historyState = getZustand();
      return VZ_RUHEND;
    }
    public Class aktivieren() {
      return VZ_ABGERECHNET;
    }
    public Class beenden() {
      return VZ_BEENDET;
    }
    public Class getZustand() {
      return VZ_ABGERECHNET;
    }
  }

  private class Abgelehnt implements Zustand {
    public Abgelehnt() {
      // damit es luft
    }
    public Class pruefen() {
      return VZ_ABGELEHNT;
    }
    public Class provisionieren() {
      return VZ_ABGELEHNT;
    }
    public Class ruhen() {
      return VZ_ABGELEHNT;
    }
    public Class aktivieren() {
      return VZ_ABGELEHNT;
    }
    public Class beenden() {
      return VZ_ABGELEHNT;
    }
    public Class getZustand() {
      return VZ_ABGELEHNT;
    }
  }
  private class Ruhend implements Zustand {
    public Ruhend() {
      // damit es luft
    }
    public Class pruefen() {
      return VZ_RUHEND;
    }
    public Class provisionieren() {
      return VZ_RUHEND;
    }
    public Class ruhen() {
      return VZ_RUHEND;
    }
    public Class aktivieren() {
      return historyState;   // historisierten Zustand wieder einnehmen
    }
    public Class beenden() {
      return VZ_RUHEND;
    }
    public Class getZustand() {
      return VZ_RUHEND;
    }
  }
  private class Beendet implements Zustand {
    public Beendet() {
      // damit es luft
    }
    public Class pruefen() {
      return VZ_BEENDET;
    }
    public Class provisionieren() {
      return VZ_BEENDET;
    }
    public Class ruhen() {
      return VZ_BEENDET;
    }
    public Class aktivieren() {
      return VZ_BEENDET;
    }
    public Class beenden() {
      return VZ_BEENDET;
    }
    public Class getZustand() {
      return VZ_BEENDET;
    }
  }


  private long nummer;
  private Zustand myZustand;

  public Vertrag(long nr) {
    myZustand = new Angelegt();
    nummer = nr;
    setStatus(VZ_ANGELEGT);
    historyState = VZ_UNGUELTIG;
  }

  public long getNummer(){
    return nummer;
  }

  public boolean pruefen() {
    setStatus(myZustand.pruefen());
    if (VZ_GEPRUEFT == myZustand.getZustand()) {
      return true;
    }
    return false;
  }

  public void abrechnen() {
    setStatus(myZustand.provisionieren());
  }

  public void ruhen() {
    setStatus(myZustand.ruhen());
  }

  public void aktivieren() {
    setStatus(myZustand.aktivieren());
  }

  public void beenden() {
    setStatus(myZustand.beenden());
  }

  public Class getZustand() {
    return myZustand.getZustand();
  }

  public boolean isAktiv() {
    if ((VZ_ANGELEGT == getZustand()) ||
        (VZ_GEPRUEFT == getZustand()) ||
        (VZ_ABGERECHNET == getZustand())) {
      return true;
    } else {
      return false;
    }
  }

  private void setStatus(Class neuerZustand) {
    try {
      Constructor cnstrctr = neuerZustand.getDeclaredConstructor(new Class[] {Vertrag.class});
      cnstrctr.setAccessible(true);
      myZustand = (Zustand) cnstrctr.newInstance(new Object[]{this});

    }
    catch (IllegalAccessException ex) {
      myZustand = (Zustand) new Ungueltig();
    }
    catch (InstantiationException ex) {
      myZustand = (Zustand) new Ungueltig();
    }
    catch (SecurityException ex1) {
      myZustand = (Zustand) new Ungueltig();
    }
    catch (NoSuchMethodException ex1) {
      myZustand = (Zustand) new Ungueltig();
    }
    catch (InvocationTargetException ex2) {
      myZustand = (Zustand) new Ungueltig();
    }
    catch (IllegalArgumentException ex2) {
      myZustand = (Zustand) new Ungueltig();
    }
  }
}